/*
 * @COPYRIGHT@
 * 
 * x-kernel v3.2
 * 
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 * 
 * @COPYRIGHT@
 *
 * $RCSfile: scout_device.c,v $
 *
 * HISTORY
 * $Log: scout_device.c,v $
 * Revision 1.4  1994/06/11  00:02:29  menze
 * Added copyright notice
 *
 * Revision 1.3  1994/05/14  23:53:13  davidm
 * (scout_device_info): changed syntax for device names.  Unit number now
 * directly follows device name.
 *
 * Revision 1.2  1994/04/13  16:13:08  davidm
 * Added RCS header.
 *
 */
/*
 * Scout device interface.
 */

#include <ctype.h>
#include <string.h>
#include "scout_device.h"
#include "scout_malloc.h"
#include "scout_trace.h"
#include "scout_util.h"

int trace_scout_device = 0;

static Scout_Device_Info	*device_list = 0, *last_entry = 0;


void
scout_device_add(Scout_Device_Info *device_info)
{
    Scout_Device_Info *di;
    int len;

    len = strlen(device_info->name);
    di = scout_malloc(sizeof(Scout_Device_Info) + len + 1);
    if (di) {
	bcopy(device_info, di, sizeof(*di));
	strcpy((char*)(di + 1), di->name);
	di->name = (char*)(di + 1);
	di->next = 0;

	/* need to insert at end to preserve order: */
	if (last_entry) {
	    last_entry->next = di;
	} else {
	    device_list = di;
	} /* if */
	last_entry = di;

	ScoutTrace(scout_device, TR_DETAILED,
		   ("scout_device_add(name=%s,base=%p,ipl=%d,rom-offset=%d)",
		    di->name, di->base_addr, di->ipl, di->rom_offset));
    } else {
	scout_panic("scout_device_add: unable to add device info entry");
    } /* if */
} /* scout_device_add */


const Scout_Device_Info*
scout_device_info(char *device_name)
{
    char *str;
    int unit, len;
    Scout_Device_Info *di;

    /*
     * Device_name is assumed to have format: <type>[<unit#>], where
     * <unit#> is a string of ASCII digits.
     */
    unit = 0;
    len = strlen(device_name);
    str = &device_name[len];
    while (str > device_name && isdigit(str[-1])) {
	--str;
    } /* while */
    if (isdigit(*str)) {
	sscanf(str, "%d", &unit);
	len = str - device_name;
    } /* if */

    if (len >= SCOUT_MAX_DEVICE_NAME_LEN) {
	printf("scout_device_info: device name `%s' too long.\n",
	       device_name);
	return 0;
    } /* if */

    di = device_list;

    /* find the "unit"-th entry of <type>: */
    while (di) {
	if (strncmp(device_name, di->name, len) == 0 &&
	    di->name[len] == '\0' && unit-- == 0)
	{
	    return di;
	} /* if */
	di = di->next;
    } /* while */
    return 0;
} /* scout_device_info */

			/*** end of scout_device.c ***/
